--[[
	Copyright (C) 2016  Fuhrbolg - Hyjal

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>

	Author's note: Under no circumstances should this Addon be used under any situation.
	It is dangerous and will cause significant harm to your computer. You have been warned.
--]]

--[[
 -	Import variables from other MongoMon source files here.
--]]
local ADDON_NAME, T = ...

--[[
 -	Library verification follows.
--]]
if not LibStub then
    error(ADDON_NAME, " requires LibStub")
    return
end

if not LibStub("AceAddon-3.0") then
    error(ADDON_NAME, " requires AceAddon-3.0")
    return
end

if not LibStub("AceGUI-3.0") then
    error(ADDON_NAME, " requires AceGUI-3.0")
    return
end

if not LibStub("AceLocale-3.0") then
    error(ADDON_NAME, " requires AceLocale-3.0")
    return
end

if not LibStub("CallbackHandler-1.0") then
    error(ADDON_NAME, " requires CallbackHandler-1.0")
    return
end

if not LibStub("LibNameplateRegistry-1.0") then
    error(ADDON_NAME, " requires LibNameplateRegistry-1.0")
    return
end

if not LibStub("LibButtonGlow-1.0") then
	error(ADDON_NAME, " requires LibButtonGlow-1.0")
	return
end

if not LibStub("AceConfig-3.0") then
	error(ADDON_NAME, " requires AceConfig-3.0")
	return
end

if not LibStub("AceDB-3.0") then
	error(ADDON_NAME, " requires AceDB-3.0")
	return
end

if not LibStub("AceConsole-3.0") then
	error(ADDON_NAME, " requires AceConsole-3.0")
	return
end

if not LibStub("LibGraph-2.0") then
	error(ADDON_NAME, " requires LibGraph-2.0")
	return
end

local L = LibStub("AceLocale-3.0"):GetLocale("MongoMon")

--[[
 -	This file contains global configuration variables that can be used
 -	elsewhere in the application that govern its behavior.
--]]

-- Only reward mongos whose rank is below this. We currently only have 5 portraits so.
local MAX_PLAYER_RANK = 5

-- Only display ranks from this value and below on nameplates. Too much clutter.
local MAX_NAMEPLATE_RANK = 3

-- Array of our sound files, played when target dies, praise be unto Hillary
local SOUND_FILES = {
	"Interface\\AddOns\\MongoMon\\Res\\Wunderbar.mp3",
	"Interface\\AddOns\\MongoMon\\Res\\GoodGame.mp3",
	"Interface\\AddOns\\MongoMon\\Res\\Sorry.mp3",
	"Interface\\AddOns\\MongoMon\\Res\\Excellent.mp3",
	"Interface\\AddOns\\MongoMon\\Res\\Jahahaha.mp3",
	"Interface\\AddOns\\MongoMon\\Res\\AufWiedersehen.mp3",
	"Interface\\AddOns\\MongoMon\\Res\\EnemyWeakened.mp3",
	"Interface\\AddOns\\MongoMon\\Res\\GreatShot.mp3",
	"Interface\\AddOns\\MongoMon\\Res\\JaWohl.mp3",
	"Interface\\AddOns\\MongoMon\\Res\\AllClear.mp3",
	"Interface\\AddOns\\MongoMon\\Res\\PathCleared.mp3",
	"Interface\\AddOns\\MongoMon\\Res\\Halo.mp3",
	"Interface\\AddOns\\MongoMon\\Res\\Nein.mp3",
	"Interface\\AddOns\\MongoMon\\Res\\JaWohl2.mp3"
}

-- Array of our terrible portraits
local IMAGE_FILES = {
	["Rogue"] = {
		["Assassination"] = "Interface\\AddOns\\MongoMon\\Res\\Broseph",
		["Outlaw"] = "Interface\\AddOns\\MongoMon\\Res\\Broseph",
		["Subtlety"] = "Interface\\AddOns\\MongoMon\\Res\\Broseph"
	},
	["Death Knight"] = {
		["Blood"] = "Interface\\AddOns\\MongoMon\\Res\\Emo",
		["Frost"] = "Interface\\AddOns\\MongoMon\\Res\\Emo",
		["Unholy"] = "Interface\\AddOns\\MongoMon\\Res\\Emo"
	},
	["Warrior"] = {
		["Fury"] = "Interface\\AddOns\\MongoMon\\Res\\MadBaby",
		["Arms"] = "Interface\\AddOns\\MongoMon\\Res\\MadBaby",
		["Protection"] = "Interface\\AddOns\\MongoMon\\Res\\MadBaby"
	},
	["Hunter"] = {
		["Survival"] = "Interface\\AddOns\\MongoMon\\Res\\DerpFace",
		["Beast Mastery"] = "Interface\\AddOns\\MongoMon\\Res\\DerpFace",
		["Marksmanship"] = "Interface\\AddOns\\MongoMon\\Res\\DerpFace"
	},
	["Priest"] = {
		["Shadow"] = "Interface\\AddOns\\MongoMon\\Res\\Kitten"
	},
	["Paladin"] = {
		["Retribution"] = "Interface\\AddOns\\MongoMon\\Res\\AngryGirl"
	},
	["Druid"] = {
		["Restoration"] = "Interface\\AddOns\\MongoMon\\Res\\CatNerd",
		["Feral"] = "Interface\\AddOns\\MongoMon\\Res\\CatLady",
		["Balance"] = "Interface\\AddOns\\MongoMon\\Res\\Hillary"
	},
	["Warlock"] = {
		["Affliction"] = "Interface\\AddOns\\MongoMon\\Res\\Impossibru",
		["Demonology"] = "Interface\\AddOns\\MongoMon\\Res\\Impossibru",
		["Destruction"] = "Interface\\AddOns\\MongoMon\\Res\\Impossibru"
	},
	["Demon Hunter"] = {
		["Havoc"] = "Interface\\AddOns\\MongoMon\\Res\\WhiskeyTango",
		["Vengeance"] = "Interface\\AddOns\\MongoMon\\Res\\WhiskeyTango"
	},
	["Monk"] = {
		["Windwalker"] = "Interface\\AddOns\\MongoMon\\Res\\Terminator"
	}
}

-- Just jesus christ { left, right, top, bottom }
local SPEC_ICONS = {
	["Death Knight-Blood"] = { 0, 1/9, 0, 1/5 },
	["Death Knight-Frost"] = { 0, 1/9, 1/5, 2/5 },
	["Death Knight-Unholy"] = { 0, 1/9, 2/5, 3/5 },
	["Demon Hunter-Havoc"] = { 0, 1/9, 3/5, 4/5 },
	["Demon Hunter-Vengeance"] = { 0, 1/9, 4/5, 5/5 },
	["Druid-Balance"] = { 1/9, 2/9, 0, 1/5 },
	["Druid-Feral"] = { 1/9, 2/9, 1/5, 2/5 },
	["Druid-Guardian"] = { 1/9, 2/9, 2/5, 3/5 },
	["Druid-Restoration"] = { 1/9, 2/9, 3/5, 4/5 },
	["Hunter-Beast Mastery"] = { 1/9, 2/9, 4/5, 5/5 },
	["Hunter-Marksmanship"] = { 2/9, 3/9, 0, 1/5 },
	["Hunter-Survival"] = { 2/9, 3/9, 1/5, 2/5 },
	["Mage-Arcane"] = { 2/9, 3/9, 2/5, 3/5 },
	["Mage-Fire"] = { 2/9, 3/9, 3/5, 4/5 },
	["Mage-Frost"] = { 2/9, 3/9, 4/5, 5/5 },
	["Monk-Brewmaster"] = { 3/9, 4/9, 0, 1/5 },
	["Monk-Mistweaver"] = { 3/9, 4/9, 1/5, 2/5 },
	["Monk-Windwalker"] = { 3/9, 4/9, 2/5, 3/5 },
	["Paladin-Holy"] = { 3/9, 4/9, 3/5, 4/5 },
	["Paladin-Protection"] = { 3/9, 4/9, 4/5, 5/5 },
	["Paladin-Retribution"] = { 4/9, 5/9, 0, 1/5 },
	["Rogue-Assassination"] = { 4/9, 5/9, 1/5, 2/5 },
	["Rogue-Outlaw"] = { 4/9, 5/9, 2/5, 3/5 },
	["Rogue-Subtlety"] = { 4/9, 5/9, 3/5, 4/5 },
	["Shaman-Elemental"] = { 4/9, 5/9, 4/5, 5/5 },
	["Priest-Discipline"] = { 5/9, 6/9, 0, 1/5 },
	["Shaman-Enhancement"] = { 5/9, 6/9, 1/5, 2/5 },
	["Warlock-Demonology"] = { 5/9, 6/9, 2/5, 3/5 },
	["Warlock-Destruction"] = { 5/9, 6/9, 3/5, 4/5 },
	["Warrior-Arms"] = { 5/9, 6/9, 4/5, 5/5 },
	["Priest-Holy"] = { 6/9, 7/9, 0, 1/5 },
	["Shaman-Restoration"] = { 6/9, 7/9, 1/5, 2/5 },
	["Warrior-Fury"] = { 6/9, 7/9, 2/5, 3/5 }, 
	["Priest-Shadow"] = { 7/9, 8/9, 0, 1/5 },
	["Warlock-Affliction"] = { 7/9, 8/9, 1/5, 2/5 },
	["Warrior-Protection"] = { 7/9, 8/9, 2/5, 3/5 }
}

-- Class specific colors http://wowwiki.wikia.com/wiki/Class_colors
local CLASS_COLORS = {
	["Death Knight"] = { 0.77, 0.12, 0.23 },
	["Demon Hunter"] = { 0.64, 0.19, 0.79 },
	["Druid"] = { 1.0, 0.49, 0.04 },
	["Hunter"] = { 0.67, 0.83, 0.45 },
	["Mage"] = { 0.41, 0.80, 0.94 },
	["Monk"] = { 0.33, 0.54, 0.52 },
	["Paladin"] = { 0.96, 0.55, 0.73 },
	["Priest"] = { 1.0, 1.0, 1.0 },
	["Rogue"] = { 1.0, 0.96, 0.41 },
	["Shaman"] = { 0.0, 0.44, 0.87 },
	["Warlock"] = { 0.58, 0.51, 0.79 },
	["Warrior"] = { 0.78, 0.61, 0.43 }
}

-- This is the actual Map ID (not to be confused with battleground id)
-- I am using this because it is a hell of a lot easier to procure this
-- tiny piece of information than get the stupid cocksucking battleground id.
-- http://wow.gamepedia.com/MapID
local BG_MAP_IDS = {
	[401] = "Alterac Valley",
	[461] = "Arathi Basin",
	[935] = "Deepwind Gorge",
	[482] = "Eye of the Storm",
	[540] = "Isle of Conquest",
	[860] = "Silvershard Mines",
	[512] = "Strand of the Ancients",
	[856] = "Temple of Kotmogu",
	[736] = "The Battle for Gilneas",
	[626] = "Twin Peaks",
	[443] = "Warsong Gulch"
}

-- Needed to map the codes for the GetBattlefieldWinners API
-- Which is silly because the battlefield stats updates use the string representation
local FACTIONS = {
	[0] = "Horde",
	[1] = "Alliance"
}

-- Only store this many MatchHistoryRecord objects at any given time. Purge oldest record.
local MAX_BG_HISTORY = 30

-- Used to determine whether player is a Healer or DPS
local HEALER_SPECS = { "Discipline", "Holy", "Restoration", "Mistweaver" }

-- Matches any "unit" under the player's control, used by the COMBAT_LOG_UNFILTERED event handler to look for killing blows
-- borrowed from the KillingBlow_Enhanced addon. Muchos gracious.
local FILTER_MINE = bit.bor(COMBATLOG_OBJECT_AFFILIATION_MINE, COMBATLOG_OBJECT_REACTION_FRIENDLY, COMBATLOG_OBJECT_CONTROL_PLAYER)

-- Players can only send the after action report to chat once per this many seconds
local SEND_TO_CHAT_COOLDOWN = 60

-- Should always be disabled.
local DEBUG_ENABLED = false

--[[
 -	Export variables that may be used by other MongoMon source files.
--]]
T["MAX_PLAYER_RANK"] = MAX_PLAYER_RANK
T["MAX_NAMEPLATE_RANK"] = MAX_NAMEPLATE_RANK
T["SOUND_FILES"] = SOUND_FILES
T["IMAGE_FILES"] = IMAGE_FILES
T["SPEC_ICONS"] = SPEC_ICONS
T["CLASS_COLORS"] = CLASS_COLORS
T["BG_MAP_IDS"] = BG_MAP_IDS
T["FACTIONS"] = FACTIONS
T["MAX_BG_HISTORY"] = MAX_BG_HISTORY
T["HEALER_SPECS"] = HEALER_SPECS
T["FILTER_MINE"] = FILTER_MINE
T["SEND_TO_CHAT_COOLDOWN"] = SEND_TO_CHAT_COOLDOWN
T["DEBUG_ENABLED"] = DEBUG_ENABLED